@@ -205,6 +205,24 @@ macro_rules! parse_generics_impl {
             [$($token)*]
         }
     };
+    (
+        @param
+        [$param:tt]
+        [$callback:path]
+        [$($callback_args:tt)*]
+        [$($g:tt)*]
+        [$($r:tt)*]
+        [ >> $($token:tt)*]
+    ) => {
+        $crate::parse_generics_impl! {
+            @done
+            [$callback] [$($callback_args)*]
+            [$($g)* [$param]]
+            [$($r)* [$param]]
+            []
+            [ > $($token)*]
+        }
+    };
     (
         @param
         [$param:tt]
@@ -223,6 +241,24 @@ macro_rules! parse_generics_impl {
             [$($token)*]
         }
     };
+    (
+        @param
+        [$param:tt]
+        [$callback:path]
+        [$($callback_args:tt)*]
+        [$($g:tt)*]
+        [$($r:tt)*]
+        [ , >> $($token:tt)*]
+    ) => {
+        $crate::parse_generics_impl! {
+            @done
+            [$callback] [$($callback_args)*]
+            [$($g)* [$param]]
+            [$($r)* [$param]]
+            []
+            [ > $($token)*]
+        }
+    };
     (
         @param
         [$param:tt]
@@ -251,7 +287,7 @@ macro_rules! parse_generics_impl {
         $crate::std_compile_error!($crate::std_concat!(
             "unexpected token '",
             $crate::std_stringify!($x),
-            "', expected ':', ',', or '>'"
+            "', expected ':', '=', ',', or '>'"
         ));
     };
     (
@@ -288,6 +324,25 @@ macro_rules! parse_generics_impl {
             [$($token)*]
         }
     };
+    (
+        @constrained_param [$kind:tt]
+        [$param:tt]
+        [$($constraint:tt)*]
+        [$callback:path]
+        [$($callback_args:tt)*]
+        [$($g:tt)*]
+        [$($r:tt)*]
+        [ << $($token:tt)*]
+    ) => {
+        $crate::parse_generics_impl! {
+            @angles_in_constraint [$kind]
+            [$param]
+            [$($constraint)*]
+            [] []
+            [$callback] [$($callback_args)*] [$($g)*] [$($r)*]
+            [ < $($token)*]
+        }
+    };
     (
         @constrained_param [:]
         [$param:tt]
@@ -307,6 +362,25 @@ macro_rules! parse_generics_impl {
             [$($token)*]
         }
     };
+    (
+        @constrained_param [:]
+        [$param:tt]
+        [$($constraint:tt)*]
+        [$callback:path]
+        [$($callback_args:tt)*]
+        [$($g:tt)*]
+        [$($r:tt)*]
+        [ >> $($token:tt)*]
+    ) => {
+        $crate::parse_generics_impl! {
+            @done
+            [$callback] [$($callback_args)*]
+            [$($g)* [$param : $($constraint)*]]
+            [$($r)* [$param]]
+            []
+            [ > $($token)*]
+        }
+    };
     (
         @constrained_param [=]
         [$param:tt]
@@ -326,6 +400,25 @@ macro_rules! parse_generics_impl {
             [$($token)*]
         }
     };
+    (
+        @constrained_param [=]
+        [$param:tt]
+        [$($constraint:tt)*]
+        [$callback:path]
+        [$($callback_args:tt)*]
+        [$($g:tt)*]
+        [$($r:tt)*]
+        [ >> $($token:tt)*]
+    ) => {
+        $crate::parse_generics_impl! {
+            @done
+            [$callback] [$($callback_args)*]
+            [$($g)* [$param]]
+            [$($r)* [$param]]
+            []
+            [ > $($token)*]
+        }
+    };
     (
         @constrained_param [:]
         [$param:tt]
@@ -345,6 +438,25 @@ macro_rules! parse_generics_impl {
             [$($token)*]
         }
     };
+    (
+        @constrained_param [:]
+        [$param:tt]
+        [$($constraint:tt)*]
+        [$callback:path]
+        [$($callback_args:tt)*]
+        [$($g:tt)*]
+        [$($r:tt)*]
+        [ , >> $($token:tt)*]
+    ) => {
+        $crate::parse_generics_impl! {
+            @done
+            [$callback] [$($callback_args)*]
+            [$($g)* [$param : $($constraint)*]]
+            [$($r)* [$param]]
+            []
+            [ > $($token)*]
+        }
+    };
     (
         @constrained_param [=]
         [$param:tt]
@@ -364,6 +476,25 @@ macro_rules! parse_generics_impl {
             [$($token)*]
         }
     };
+    (
+        @constrained_param [=]
+        [$param:tt]
+        [$($constraint:tt)*]
+        [$callback:path]
+        [$($callback_args:tt)*]
+        [$($g:tt)*]
+        [$($r:tt)*]
+        [ , >> $($token:tt)*]
+    ) => {
+        $crate::parse_generics_impl! {
+            @done
+            [$callback] [$($callback_args)*]
+            [$($g)* [$param]]
+            [$($r)* [$param]]
+            []
+            [ > $($token)*]
+        }
+    };
     (
         @constrained_param [:]
         [$param:tt]
@@ -448,10 +579,30 @@ macro_rules! parse_generics_impl {
             @constrained_param [$kind]
             [$param]
             [$($constraint)* < $($inside_angles)* > ]
-            [$callback] [$($callback_args)*] [$($g)*] [$($r)*] 
+            [$callback] [$($callback_args)*] [$($g)*] [$($r)*]
             [$($token)*]
         }
     };
+    (
+        @angles_in_constraint [$kind:tt]
+        [$param:tt]
+        [$($constraint:tt)*]
+        [$($inside_angles:tt)*]
+        []
+        [$callback:path]
+        [$($callback_args:tt)*]
+        [$($g:tt)*]
+        [$($r:tt)*]
+        [ >> $($token:tt)*]
+    ) => {
+        $crate::parse_generics_impl! {
+            @constrained_param [$kind]
+            [$param]
+            [$($constraint)* < $($inside_angles)* > ]
+            [$callback] [$($callback_args)*] [$($g)*] [$($r)*] 
+            [ > $($token)*]
+        }
+    };
     (
         @angles_in_constraint [$kind:tt]
         [$param:tt]
@@ -469,10 +620,31 @@ macro_rules! parse_generics_impl {
             [$param] [$($constraint)*]
             [$($parent_level)* < $($inside_angles)* > ]
             [$([$($outer_levels)*])*]
-            [$callback] [$($callback_args)*] [$($g)*] [$($r)*] 
+            [$callback] [$($callback_args)*] [$($g)*] [$($r)*]
             [$($token)*]
         }
     };
+    (
+        @angles_in_constraint [$kind:tt]
+        [$param:tt]
+        [$($constraint:tt)*]
+        [$($inside_angles:tt)*]
+        [[$($parent_level:tt)*] $([$($outer_levels:tt)*])*]
+        [$callback:path]
+        [$($callback_args:tt)*]
+        [$($g:tt)*]
+        [$($r:tt)*]
+        [ >> $($token:tt)*]
+    ) => {
+        $crate::parse_generics_impl! {
+            @angles_in_constraint [$kind]
+            [$param] [$($constraint)*]
+            [$($parent_level)* < $($inside_angles)* > ]
+            [$([$($outer_levels)*])*]
+            [$callback] [$($callback_args)*] [$($g)*] [$($r)*] 
+            [ > $($token)*]
+        }
+    };
     (
         @angles_in_constraint [$kind:tt]
         [$param:tt]
@@ -490,10 +662,31 @@ macro_rules! parse_generics_impl {
             [$param] [$($constraint)*]
             []
             [[$($inside_angles:tt)*] $([$($outer_levels)*])*]
-            [$callback] [$($callback_args)*] [$($g)*] [$($r)*] 
+            [$callback] [$($callback_args)*] [$($g)*] [$($r)*]
             [$($token)*]
         }
     };
+    (
+        @angles_in_constraint [$kind:tt]
+        [$param:tt]
+        [$($constraint:tt)*]
+        [$($inside_angles:tt)*]
+        [$([$($outer_levels:tt)*])*]
+        [$callback:path]
+        [$($callback_args:tt)*]
+        [$($g:tt)*]
+        [$($r:tt)*]
+        [ << $($token:tt)*]
+    ) => {
+        $crate::parse_generics_impl! {
+            @angles_in_constraint [$kind]
+            [$param] [$($constraint)*]
+            []
+            [[$($inside_angles:tt)*] $([$($outer_levels)*])*]
+            [$callback] [$($callback_args)*] [$($g)*] [$($r)*] 
+            [ < $($token)*]
+        }
+    };
     (
         @angles_in_constraint [$kind:tt]
         [$param:tt]
